* Output options: all can be turned on/off by adding or removing the NO;
* page number, date, centering, or page breaks, page length and width;
OPTIONS NOnumber NOdate NOcenter FormDlim=" " PageSize=MAX LineSize=MAX SPOOL;
* Eliminate SAS default titles and names of tables in output (TRACE ON to show);
TITLE; ODS TRACE OFF; *ODS LISTING;

***********************************************************************************;
*******     MACRO PROGRAMS TO AUTOMATE CALCULATIONS FOR THIS EXAMPLE        *******;
*******               NOTHING IN HERE NEEDS TO BE CHANGED                   *******;
***********************************************************************************;

* To use PseudoR2 macro;
* Ncov =     TOTAL # entries in covariance parameter estimates table;
* CovFewer = Name of ODS CovParms table for nested baseline model (required);
* CovMore1 = Name of ODS CovParms table for comparison model1 (required);
* CovMore2 = Name of ODS CovParms table for comparison model2 (optional);
%MACRO PseudoR2(NCov=, CovFewer=, CovMore1=, CovMore2=);
DATA &CovFewer.; LENGTH Name $30.; SET &CovFewer.; Name="&CovFewer."; RUN;
DATA &CovMore1.; LENGTH Name $30.; SET &CovMore1.; Name="&CovMore1."; RUN;
DATA CovCompare1; LENGTH Name $30.; SET &CovFewer. &CovMore1.; RUN;
DATA CovCompare1; SET CovCompare1; 
     PseudoR2=(LAG&Ncov.(Estimate)-Estimate)/LAG&Ncov.(Estimate); RUN;
%IF %LENGTH(&CovMore2.)>0 %THEN %DO; %LET lagchange=%EVAL(&Ncov.*2);
DATA &CovMore2.; LENGTH Name $30.; SET &CovMore2.; Name="&CovMore2."; RUN;
DATA CovCompare2; LENGTH Name $30.; SET &CovFewer. &CovMore2.; RUN;
DATA CovCompare2; SET CovCompare2; 
     PseudoR2=(LAG&Ncov.(Estimate)-Estimate)/LAG&Ncov.(Estimate); RUN;
DATA CovCompare; SET CovCompare1 CovCompare2;
     PseudoR2Change=PseudoR2-(LAG&lagchange(PseudoR2)); RUN;
%END; %IF %LENGTH(&CovMore2.)>0 %THEN %LET ToPrint=CovCompare; 
     %ELSE %LET ToPrint=CovCompare1;
DATA &ToPrint.; SET &ToPrint.; 
     IF CovParm IN("UN(2,1)","UN(3,1)","UN(3,2)","UN(4,1)","UN(4,2)","UN(4,3)",
                   "UN(5,1)","UN(5,2)","UN(5,3)","UN(5,4)",
                   "UN(6,1)","UN(6,2)","UN(6,3)","UN(6,4)","UN(6,5)") THEN DELETE; 
     DROP StdErr Gradient Zvalue ProbZ; RUN;
TITLE8 "Pseudo-R2 Relative to &CovFewer.";
%IF %LENGTH(&CovMore2.)>0 %THEN %DO;
TITLE9 "Change in Pseudo-R2 for &CovMore1. vs. &CovMore2."; %END;
PROC PRINT NOOBS DATA=&ToPrint.; RUN; TITLE8; TITLE9;
%MEND PseudoR2;


***********************************************************************************;
*******           BEGIN DATA IMPORT AND PREPARATION FOR EXAMPLE 6a          *******;
*******               CHANGE "filesave" TO YOUR FOLDER PATH                 *******;
***********************************************************************************;

* Define global variable for file location to be replaced in code below;
* \\Client\ precedes actual path when using UIowa Virtual Desktop;
%LET filesave=C:\Dropbox\23_PSQF6272\PSQF6272_Example6a;

* Import trimmed example data excel data file from sheet "grade10" into SAS;
PROC IMPORT DATAFILE="&filesave.\Example6a_Data.xlsx" OUT=work.Example6a DBMS=XLSX REPLACE; 
     SHEET="grade10"; GETNAMES=YES; RUN;

* Open external file to save results to;
ODS RTF FILE="&filesave.\Example6a_SAS_Output.rtf" 
        STYLE=HTMLBlue STARTPAGE=NO BODYTITLE;

TITLE "SAS Descriptive Statistics within Student-Level Data";
PROC FREQ DATA=work.Example6a; TABLE lunch; RUN;
PROC MEANS NDEC=2 DATA=work.Example6a; VAR math; RUN;

* Data preparation;
DATA work.Example6a; SET work.Example6a; 
* Add labels to original variables;
  LABEL districtID= "districtID: District ID number"
        studentID=  "studentID: Student ID number"
        schoolID=   "schoolID: School ID number"
        lunch=      "lunch: 0=Paid, 1=Reduced, 2=Free"
        math=       "math: Math Test Score";
* Create new lunch variables for clarity;
  lunch3 = lunch; * 3-category version;
  IF lunch>0 THEN lunch2=1; ELSE lunch2=lunch; * 2-category version; 
* Filter to complete cases before computing cluster means;
  IF NMISS(math,lunch)>0 THEN DELETE;
RUN;

* Create cluster mean math;
PROC SORT DATA=work.Example6a; BY schoolID; RUN;
PROC MEANS NOPRINT N DATA=work.Example6a; BY schoolID; VAR math;
     OUTPUT OUT=work.SchoolMeans MEAN(math)= CMmath; RUN;

* Get sample size per school and label new school mean variables;
DATA work.SchoolMeans; SET work.SchoolMeans;
     schoolN=_FREQ_; DROP _TYPE_ _FREQ_;
     LABEL schoolN= "schoolN: # Students Sampled Per School"
           CMmath=  "CMmath: School Mean Math Score"; RUN;

TITLE "SAS Descriptive Statistics within School-Level Data";
PROC MEANS NDEC=2 DATA=work.SchoolMeans; VAR schoolN CMmath; RUN; TITLE;

* Merge school means back into student data and center predictors;
DATA work.Example6a; MERGE work.Example6a work.SchoolMeans; BY schoolID; 
* Rescale and center cluster mean math to be per 10 points;
  CMmath50 = (CMmath-50)/10;
  LABEL CMmath50 = "CMmath50: School Mean Math (0=50)";
* Rescale and cluster-MC student math to be per 10 points;
  WCmath = (math-CMmath)/10; 
  LABEL WCmath= "WCmath: Within-School Math (0=CM)"; RUN;

TITLE "SAS Descriptive Statistics within Student-Level Data";
PROC MEANS NOLABELS NDEC=3 N MEAN VAR STDDEV MIN MAX DATA=work.Example6a; 
     VAR WCmath; RUN; TITLE;


***********************************************************************************;
*******                   BEGIN BINARY MODELS FOR EXAMPLE 6a                *******;
***********************************************************************************;

* NOTE: I am using no DDFM and 7 points of integration to match STATA default;

TITLE "SAS Model 1: Empty Means, Single-Level for Student Binary Lunch";
PROC GLIMMIX DATA=work.Example6a NOCLPRINT NAMELEN=100 METHOD=QUAD(QPOINTS=7) GRADIENT;
     CLASS schoolID; * Descending makes us predict the 1 instead of the 0;
     MODEL lunch2 (DESCENDING) =  / SOLUTION LINK=LOGIT DIST=BINARY DDFM=NONE;
     ESTIMATE "Intercept" intercept 1 / ILINK; * ILINK is inverse link (to un-logit);
RUN; TITLE;

TITLE "SAS Model 2: Empty Means, Random Intercept for Student Binary Lunch";
PROC GLIMMIX DATA=work.Example6a NOCLPRINT NAMELEN=100 METHOD=QUAD(QPOINTS=7) GRADIENT;
     CLASS schoolID;
     MODEL lunch2 (DESCENDING) =  / SOLUTION LINK=LOGIT DIST=BINARY DDFM=NONE;
     RANDOM INTERCEPT / TYPE=UN SUBJECT=schoolID;
     ESTIMATE "Intercept" intercept 1 / ILINK; * ILINK is inverse link (to un-logit);
     COVTEST "Random School Intercept?" 0;     * Test if G matrix (1,1)=0;
     ODS OUTPUT CovParms=CovEmpty;             * Save random int var for pseudo-R2;
RUN; TITLE;

TITLE "SAS Model 3: Add Level-2 Fixed Slope of School Mean Math";
PROC GLIMMIX DATA=work.Example6a NOCLPRINT NAMELEN=100 METHOD=QUAD(QPOINTS=7) GRADIENT;
     CLASS schoolID;
     MODEL lunch2 (DESCENDING) = CMmath50 / SOLUTION LINK=LOGIT DIST=BINARY DDFM=NONE ODDSRATIO;
     RANDOM INTERCEPT / TYPE=UN SUBJECT=schoolID;
     ESTIMATE "Intercept if SMmath=49"  intercept 1 CMmath50 -1 / ILINK; 
     ESTIMATE "Intercept if SMmath=50"  intercept 1 CMmath50  0 / ILINK; 
     ESTIMATE "Intercept if SMmath=51"  intercept 1 CMmath50  1 / ILINK; 
     ESTIMATE "L2 Math Slope"           CMmath50 1 / ILINK; * Example of non-sense ILINK;
     ODS OUTPUT CovParms=CovCMmath;     * Save random int var for pseudo-R2;
RUN; TITLE;
* Calculate PseudoR2 relative to empty model;
%PseudoR2(NCov=1, CovFewer=CovEmpty, CovMore1=CovCMmath);

TITLE "SAS Model 4: Add Level-1 Fixed Slope of Cluster-MC Student Math";
PROC GLIMMIX DATA=work.Example6a NOCLPRINT NAMELEN=100 METHOD=QUAD(QPOINTS=7) GRADIENT;
     CLASS schoolID;
     MODEL lunch2 (DESCENDING) = CMmath50 WCmath / SOLUTION LINK=LOGIT DIST=BINARY DDFM=NONE ODDSRATIO;
     RANDOM INTERCEPT / TYPE=UN SUBJECT=schoolID; 
     ESTIMATE "Math Contextual Slope"  WCmath -1 CMmath50 1 /EXP;
     ODS OUTPUT CovParms=CovWCmath;    * Save random int var for pseudo-R2;
RUN; TITLE;
* Calculate PseudoR2 relative to empty model;
%PseudoR2(NCov=1, CovFewer=CovEmpty, CovMore1=CovCMmath, CovMore2=CovWCmath);

TITLE "SAS Model 5: Add Random Slope of Cluster-MC Student Math";
PROC GLIMMIX DATA=work.Example6a NOCLPRINT NAMELEN=100 METHOD=QUAD(QPOINTS=7) GRADIENT;
     CLASS schoolID;
     MODEL lunch2 (DESCENDING) = CMmath50 WCmath / SOLUTION LINK=LOGIT DIST=BINARY DDFM=NONE ODDSRATIO;
     RANDOM INTERCEPT WCmath / GCORR TYPE=UN SUBJECT=schoolID; 
     COVTEST "Random Student Math Slope?" . 0 0; * Leave (1,1), test if (2,1) and (2,2) =0;
     ODS OUTPUT CovParms=CovRandMath;   * Save random variances for pseudo-R2;
RUN; TITLE;

TITLE "SAS Model 6: Add Interactions of School Mean and Cluster-MC Student Math";
PROC GLIMMIX DATA=work.Example6a NOCLPRINT NAMELEN=100 METHOD=QUAD(QPOINTS=7) GRADIENT;
     CLASS schoolID;
     MODEL lunch2 (DESCENDING) = CMmath50 WCmath CMmath50*WCmath CMmath50*CMmath50 
                       / SOLUTION LINK=LOGIT DIST=BINARY DDFM=NONE ODDSRATIO;
     RANDOM INTERCEPT WCmath / TYPE=UN SUBJECT=schoolID;
     ESTIMATE "Math Contextual Simple Main Effect"      WCmath -1 CMmath50 1 / EXP; 
     ESTIMATE "Math Contextual Interaction"    CMmath50*WCmath -1 CMmath50*CMmath50 1 / EXP;
     ODS OUTPUT CovParms=CovInteract;   * Save random variances for pseudo-R2;
RUN; TITLE;
* Calculate PseudoR2 relative to previous model 5;
%PseudoR2(NCov=3, CovFewer=CovRandMath, CovMore1=CovInteract);


***********************************************************************************;
*******                 BEGIN ORDINAL MODELS FOR EXAMPLE 6a                 *******;
***********************************************************************************;

* NOTE: I am using no DDFM and 7 points of integration to match STATA default;
* Add (DESCENDING) to get fixed intercepts instead of fixed thresholds;

TITLE "SAS Model 7: Empty Means, Random Intercept for Student Ordinal Lunch";
PROC GLIMMIX DATA=work.Example6a NOCLPRINT NAMELEN=100 METHOD=QUAD(QPOINTS=7) GRADIENT;
     CLASS schoolID;
     MODEL lunch3 =  / SOLUTION LINK=CLOGIT DIST=MULT DDFM=NONE;
     ESTIMATE "Threshold for 0 vs 12" intercept 1 0 / ILINK; * ILINK is inverse link (to un-logit);
     ESTIMATE "Threshold for 01 vs 2" intercept 0 1 / ILINK; 
RUN; TITLE;

TITLE "SAS Model 8: Empty Means, Random Intercept for Student Ordinal Lunch";
PROC GLIMMIX DATA=work.Example6a NOCLPRINT NAMELEN=100 METHOD=QUAD(QPOINTS=7) GRADIENT;
     CLASS schoolID;
     MODEL lunch3 =  / SOLUTION LINK=CLOGIT DIST=MULT DDFM=NONE;
     RANDOM INTERCEPT / TYPE=UN SUBJECT=schoolID;
     ESTIMATE "Threshold for 0 vs 12" intercept 1 0 / ILINK; * ILINK is inverse link (to un-logit);
     ESTIMATE "Threshold for 01 vs 2" intercept 0 1 / ILINK; 
     COVTEST "Random School Intercept?" 0;     * Test if G matrix (1,1)=0;
RUN; TITLE;

TITLE "SAS Model 9: Add CMmath and Fixed Slope of Cluster-MC Student Math";
PROC GLIMMIX DATA=work.Example6a NOCLPRINT NAMELEN=100 METHOD=QUAD(QPOINTS=7) GRADIENT;
     CLASS schoolID;
     MODEL lunch3 = CMmath50 WCmath / SOLUTION LINK=CLOGIT DIST=MULT DDFM=NONE ODDSRATIO;
     RANDOM INTERCEPT / TYPE=UN SUBJECT=schoolID; 
     ESTIMATE "Math Contextual Effect"  WCmath -1 CMmath50 1 / EXP;
RUN; TITLE;

TITLE "SAS Model 10: Add Random Slope of Cluster-MC Student Math";
PROC GLIMMIX DATA=work.Example6a NOCLPRINT NAMELEN=100 METHOD=QUAD(QPOINTS=7) GRADIENT;
     CLASS schoolID;
     MODEL lunch3 = CMmath50 WCmath / SOLUTION LINK=CLOGIT DIST=MULT DDFM=NONE ODDSRATIO;
     RANDOM INTERCEPT WCmath / GCORR TYPE=UN SUBJECT=schoolID; 
     COVTEST "Random Student Math Slope?" . 0 0; * Leave (1,1), test if (2,1) and (2,2) =0;
RUN; TITLE;


***********************************************************************************;
*******                 BEGIN NOMINAL MODELS FOR EXAMPLE 6a                 *******;
***********************************************************************************;

* NOTE: PROC GLIMMIX does not estimate covariances among random effects across submodels
  which is likely insufficient. You would need to use NLMIXED instead for those;
* Use METHOD=LAPLACE instead of quadrature if needed for non-convergence;

TITLE1 "SAS Model 11: Empty Means, Single-Level for Student Nominal Lunch";
TITLE2 "Model gives conditional probability for 1 instead of 0 and 2 instead of 0";
PROC GLIMMIX DATA=work.Example6a NOCLPRINT NAMELEN=100 METHOD=QUAD(QPOINTS=7) GRADIENT;
     CLASS schoolID lunch3;
     MODEL lunch3 (REFERENCE="0") =  / SOLUTION LINK=GLOGIT DIST=MULT DDFM=NONE;
     ESTIMATE "Intercept" intercept 1 / ILINK BYCAT; * ILINK is inverse link (to un-logit);
RUN; TITLE1; TITLE2;

TITLE "SAS Model 12: Empty Means, Random Intercept for Student Nominal Lunch";
PROC GLIMMIX DATA=work.Example6a NOCLPRINT NAMELEN=100 METHOD=QUAD(QPOINTS=7) GRADIENT;
     CLASS schoolID lunch3;
     MODEL lunch3 (REFERENCE="0") =  / SOLUTION LINK=GLOGIT DIST=MULT DDFM=NONE;
     RANDOM INTERCEPT / TYPE=UN SUBJECT=schoolID GROUP=lunch3;
     ESTIMATE "Intercept" intercept 1 / ILINK BYCAT; * ILINK is inverse link (to un-logit);
     COVTEST "Random School Intercept?" GLM; * Test if random intercepts needed;
RUN; TITLE;

TITLE "SAS Model 13: Add CMmath and Fixed Slope of Cluster-MC Student Math";
PROC GLIMMIX DATA=work.Example6a NOCLPRINT NAMELEN=100 METHOD=QUAD(QPOINTS=7) GRADIENT;
     CLASS schoolID lunch3;
     MODEL lunch3 (REFERENCE="0") = CMmath50 WCmath / SOLUTION LINK=GLOGIT DIST=MULT DDFM=NONE ODDSRATIO;
     RANDOM INTERCEPT / TYPE=UN SUBJECT=schoolID GROUP=lunch3; 
     ESTIMATE "Math Contextual Effect"  WCmath -1 CMmath50 1 / EXP BYCAT;
RUN; TITLE;

TITLE1 "SAS Model 14a: Add Random Slope of Cluster-MC Student Math";
TITLE2 "Does converge using 7 quadrature points";
PROC GLIMMIX DATA=work.Example6a NOCLPRINT NAMELEN=100 METHOD=QUAD(QPOINTS=7) GRADIENT;
     CLASS schoolID lunch3;
     MODEL lunch3 (REFERENCE="0") = CMmath50 WCmath / SOLUTION LINK=GLOGIT DIST=MULT DDFM=NONE ODDSRATIO;
     RANDOM INTERCEPT WCmath / TYPE=UN SUBJECT=schoolID GROUP=lunch3; 
     COVTEST "Random Student Math Slopes?" . 0 0 . 00; * Keep random ints, remove rest;
RUN; TITLE1; TITLE2;

TITLE1 "SAS Model 14b: Add Random Slope of Cluster-MC Student Math";
TITLE2 "Does converge using Laplace estimation instead (1 quadrature point)";
PROC GLIMMIX DATA=work.Example6a NOCLPRINT NAMELEN=100 METHOD=LAPLACE GRADIENT;
     CLASS schoolID lunch3;
     MODEL lunch3 (REFERENCE="0") = CMmath50 WCmath / SOLUTION LINK=GLOGIT DIST=MULT DDFM=NONE ODDSRATIO;
     RANDOM INTERCEPT WCmath / TYPE=UN SUBJECT=schoolID GROUP=lunch3; 
     COVTEST "Need random Student Math Slopes?" . 0 0 . 0 0; * Keep random ints, remove rest;
RUN; TITLE1; TITLE2;


***********************************************************************************;
* Close external file of saved results;
ODS RTF CLOSE;
